@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2016 The OpenLDAP Foundation.
+ * Copyright 1998-2020 The OpenLDAP Foundation.
  * Portions Copyright 2003 Kurt D. Zeilenga.
  * Portions Copyright 2003 IBM Corporation.
  * All rights reserved.
@@ -1877,16 +1877,17 @@ int
 tool_check_abandon( LDAP *ld, int msgid )
 {
 	int	rc;
+	LDAPControl *sctrls[1] = { NULL };
 
 	switch ( gotintr ) {
 	case Intr_Cancel:
-		rc = ldap_cancel_s( ld, msgid, NULL, NULL );
+		rc = ldap_cancel_s( ld, msgid, sctrls, NULL );
 		fprintf( stderr, "got interrupt, cancel got %d: %s\n",
 				rc, ldap_err2string( rc ) );
 		return -1;
 
 	case Intr_Abandon:
-		rc = ldap_abandon_ext( ld, msgid, NULL, NULL );
+		rc = ldap_abandon_ext( ld, msgid, sctrls, NULL );
 		fprintf( stderr, "got interrupt, abandon got %d: %s\n",
 				rc, ldap_err2string( rc ) );
 		return -1;
