@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2015 The OpenLDAP Foundation.
+ * Copyright 2004-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -736,9 +736,7 @@ lastmod_response( Operation *op, SlapReply *rs )
 }
 
 static int
-lastmod_db_init(
-	BackendDB *be
-)
+lastmod_db_init( BackendDB *be, ConfigReply *cr )
 {
 	slap_overinst		*on = (slap_overinst *)be->bd_info;
 	lastmod_info_t		*lmi;
@@ -827,9 +825,7 @@ lastmod_db_config(
 }
 
 static int
-lastmod_db_open(
-	BackendDB *be
-)
+lastmod_db_open( BackendDB *be, ConfigReply *cr )
 {
 	slap_overinst	*on = (slap_overinst *) be->bd_info;
 	lastmod_info_t	*lmi = (lastmod_info_t *)on->on_bi.bi_private;
@@ -904,9 +900,7 @@ lastmod_db_open(
 }
 
 static int
-lastmod_db_destroy(
-	BackendDB *be
-)
+lastmod_db_destroy( BackendDB *be, ConfigReply *cr )
 {
 	slap_overinst	*on = (slap_overinst *)be->bd_info;
 	lastmod_info_t	*lmi = (lastmod_info_t *)on->on_bi.bi_private;
