@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2015 The OpenLDAP Foundation.
+ * Copyright 1998-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -128,7 +128,7 @@ ber_write(
 int
 ber_realloc( BerElement *ber, ber_len_t len )
 {
-	ber_len_t	total, offset, sos_offset;
+	ber_len_t	total, offset, sos_offset, rw_offset;
 	char		*buf;
 
 	assert( ber != NULL );
@@ -165,6 +165,7 @@ ber_realloc( BerElement *ber, ber_len_t len )
 	offset = ber->ber_ptr - buf;
 	sos_offset = ber->ber_sos_ptr ? ber->ber_sos_ptr - buf : 0;
 	/* if ber_sos_ptr != NULL, it is > ber_buf so that sos_offset > 0 */
+	rw_offset = ber->ber_rwptr ? ber->ber_rwptr - buf : 0;
 
 	buf = (char *) ber_memrealloc_x( buf, total, ber->ber_memctx );
 	if ( buf == NULL ) {
@@ -176,6 +177,8 @@ ber_realloc( BerElement *ber, ber_len_t len )
 	ber->ber_ptr = buf + offset;
 	if ( sos_offset )
 		ber->ber_sos_ptr = buf + sos_offset;
+	if ( ber->ber_rwptr )
+		ber->ber_rwptr = buf + rw_offset;
 
 	return( 0 );
 }
@@ -656,7 +659,8 @@ ber_get_next(
 		ber_slen_t to_go;
 		
 		to_go = ber->ber_end - ber->ber_rwptr;
-		assert( to_go > 0 );
+		/* unsigned/signed overflow */
+		if (to_go<0) return LBER_DEFAULT;
 		
 		sock_errset(0);
 		res = ber_int_sb_read( sb, ber->ber_rwptr, to_go );
@@ -679,7 +683,7 @@ done:
 		return (ber->ber_tag);
 	}
 
-	assert( 0 ); /* ber structure is messed up ?*/
+	/* invalid input */
 	return LBER_DEFAULT;
 }
 
