@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2015 The OpenLDAP Foundation.
+ * Copyright 1998-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -85,6 +85,96 @@ ldap_add( LDAP *ld, LDAP_CONST char *dn, LDAPMod **attrs )
 }
 
 
+BerElement *
+ldap_build_add_req(
+	LDAP *ld,
+	const char *dn,
+	LDAPMod **attrs,
+	LDAPControl **sctrls,
+	LDAPControl **cctrls,
+	ber_int_t	*msgidp )
+{
+	BerElement *ber;
+	int i, rc;
+
+	/* create a message to send */
+	if ( (ber = ldap_alloc_ber_with_options( ld )) == NULL ) {
+		return( NULL );
+	}
+
+	LDAP_NEXT_MSGID(ld, *msgidp);
+	rc = ber_printf( ber, "{it{s{", /* '}}}' */
+		*msgidp, LDAP_REQ_ADD, dn );
+
+	if ( rc == -1 ) {
+		ld->ld_errno = LDAP_ENCODING_ERROR;
+		ber_free( ber, 1 );
+		return( NULL );
+	}
+
+	/* allow attrs to be NULL ("touch"; should fail...) */
+	if ( attrs ) {
+		/* for each attribute in the entry... */
+		for ( i = 0; attrs[i] != NULL; i++ ) {
+			if ( ( attrs[i]->mod_op & LDAP_MOD_BVALUES) != 0 ) {
+				int j;
+
+				if ( attrs[i]->mod_bvalues == NULL ) {
+					ld->ld_errno = LDAP_PARAM_ERROR;
+					ber_free( ber, 1 );
+					return( NULL );
+				}
+
+				for ( j = 0; attrs[i]->mod_bvalues[ j ] != NULL; j++ ) {
+					if ( attrs[i]->mod_bvalues[ j ]->bv_val == NULL ) {
+						ld->ld_errno = LDAP_PARAM_ERROR;
+						ber_free( ber, 1 );
+						return( NULL );
+					}
+				}
+
+				rc = ber_printf( ber, "{s[V]N}", attrs[i]->mod_type,
+				    attrs[i]->mod_bvalues );
+
+			} else {
+				if ( attrs[i]->mod_values == NULL ) {
+					ld->ld_errno = LDAP_PARAM_ERROR;
+					ber_free( ber, 1 );
+					return( NULL );
+				}
+
+				rc = ber_printf( ber, "{s[v]N}", attrs[i]->mod_type,
+				    attrs[i]->mod_values );
+			}
+			if ( rc == -1 ) {
+				ld->ld_errno = LDAP_ENCODING_ERROR;
+				ber_free( ber, 1 );
+				return( NULL );
+			}
+		}
+	}
+
+	if ( ber_printf( ber, /*{{*/ "N}N}" ) == -1 ) {
+		ld->ld_errno = LDAP_ENCODING_ERROR;
+		ber_free( ber, 1 );
+		return( NULL );
+	}
+
+	/* Put Server Controls */
+	if( ldap_int_put_controls( ld, sctrls, ber ) != LDAP_SUCCESS ) {
+		ber_free( ber, 1 );
+		return( NULL );
+	}
+
+	if ( ber_printf( ber, /*{*/ "N}" ) == -1 ) {
+		ld->ld_errno = LDAP_ENCODING_ERROR;
+		ber_free( ber, 1 );
+		return( NULL );
+	}
+
+	return( ber );
+}
+
 /*
  * ldap_add_ext - initiate an ldap extended add operation.  Parameters:
  *
@@ -99,7 +189,7 @@ ldap_add( LDAP *ld, LDAP_CONST char *dn, LDAPMod **attrs )
  *	msgidp	Message ID pointer
  *
  * Example:
- *	LDAPMod	*attrs[] = { 
+ *	LDAPMod	*attrs[] = {
  *			{ 0, "cn", { "babs jensen", "babs", 0 } },
  *			{ 0, "sn", { "jensen", 0 } },
  *			{ 0, "objectClass", { "person", 0 } },
@@ -117,7 +207,7 @@ ldap_add_ext(
 	int	*msgidp )
 {
 	BerElement	*ber;
-	int		i, rc;
+	int		rc;
 	ber_int_t	id;
 
 	Debug( LDAP_DEBUG_TRACE, "ldap_add_ext\n", 0, 0, 0 );
@@ -130,81 +220,9 @@ ldap_add_ext(
 	rc = ldap_int_client_controls( ld, cctrls );
 	if( rc != LDAP_SUCCESS ) return rc;
 
-	/* create a message to send */
-	if ( (ber = ldap_alloc_ber_with_options( ld )) == NULL ) {
-		ld->ld_errno = LDAP_NO_MEMORY;
+	ber = ldap_build_add_req( ld, dn, attrs, sctrls, cctrls, &id );
+	if( !ber )
 		return ld->ld_errno;
-	}
-
-	LDAP_NEXT_MSGID(ld, id);
-	rc = ber_printf( ber, "{it{s{", /* '}}}' */
-		id, LDAP_REQ_ADD, dn );
-
-	if ( rc == -1 ) {
-		ld->ld_errno = LDAP_ENCODING_ERROR;
-		ber_free( ber, 1 );
-		return ld->ld_errno;
-	}
-
-	/* allow attrs to be NULL ("touch"; should fail...) */
-	if ( attrs ) {
-		/* for each attribute in the entry... */
-		for ( i = 0; attrs[i] != NULL; i++ ) {
-			if ( ( attrs[i]->mod_op & LDAP_MOD_BVALUES) != 0 ) {
-				int j;
-
-				if ( attrs[i]->mod_bvalues == NULL ) {
-					ld->ld_errno = LDAP_PARAM_ERROR;
-					ber_free( ber, 1 );
-					return ld->ld_errno;
-				}
-
-				for ( j = 0; attrs[i]->mod_bvalues[ j ] != NULL; j++ ) {
-					if ( attrs[i]->mod_bvalues[ j ]->bv_val == NULL ) {
-						ld->ld_errno = LDAP_PARAM_ERROR;
-						ber_free( ber, 1 );
-						return ld->ld_errno;
-					}
-				}
-
-				rc = ber_printf( ber, "{s[V]N}", attrs[i]->mod_type,
-				    attrs[i]->mod_bvalues );
-
-			} else {
-				if ( attrs[i]->mod_values == NULL ) {
-					ld->ld_errno = LDAP_PARAM_ERROR;
-					ber_free( ber, 1 );
-					return ld->ld_errno;
-				}
-
-				rc = ber_printf( ber, "{s[v]N}", attrs[i]->mod_type,
-				    attrs[i]->mod_values );
-			}
-			if ( rc == -1 ) {
-				ld->ld_errno = LDAP_ENCODING_ERROR;
-				ber_free( ber, 1 );
-				return ld->ld_errno;
-			}
-		}
-	}
-
-	if ( ber_printf( ber, /*{{*/ "N}N}" ) == -1 ) {
-		ld->ld_errno = LDAP_ENCODING_ERROR;
-		ber_free( ber, 1 );
-		return ld->ld_errno;
-	}
-
-	/* Put Server Controls */
-	if( ldap_int_put_controls( ld, sctrls, ber ) != LDAP_SUCCESS ) {
-		ber_free( ber, 1 );
-		return ld->ld_errno;
-	}
-
-	if ( ber_printf( ber, /*{*/ "N}" ) == -1 ) {
-		ld->ld_errno = LDAP_ENCODING_ERROR;
-		ber_free( ber, 1 );
-		return ld->ld_errno;
-	}
 
 	/* send the message */
 	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_ADD, dn, ber, id );
