@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2019 The OpenLDAP Foundation.
+ * Copyright 2000-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,7 @@ static int mdb_id2entry_put(
 	struct mdb_info *mdb = (struct mdb_info *) op->o_bd->be_private;
 	Ecount ec;
 	MDB_val key, data;
-	int rc;
+	int rc, prev_ads = mdb->mi_numads;
 
 	/* We only store rdns, and they go in the dn2id database. */
 
@@ -55,8 +55,10 @@ static int mdb_id2entry_put(
 	key.mv_size = sizeof(ID);
 
 	rc = mdb_entry_partsize( mdb, txn, e, &ec );
-	if (rc)
-		return LDAP_OTHER;
+	if (rc) {
+		rc = LDAP_OTHER;
+		goto fail;
+	}
 
 	flag |= MDB_RESERVE;
 
@@ -72,7 +74,7 @@ again:
 	if (rc == MDB_SUCCESS) {
 		rc = mdb_entry_encode( op, e, &data, &ec );
 		if( rc != LDAP_SUCCESS )
-			return rc;
+			goto fail;
 	}
 	if (rc) {
 		/* Was there a hole from slapadd? */
@@ -87,6 +89,10 @@ again:
 		if ( rc != MDB_KEYEXIST )
 			rc = LDAP_OTHER;
 	}
+fail:
+	if (rc) {
+		mdb_ad_unwind( mdb, prev_ads );
+	}
 	return rc;
 }
 
