@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2015 The OpenLDAP Foundation.
+ * Copyright 1998-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -94,7 +94,7 @@ int slapi_plugins_used = 0;
 static int fp_getline(FILE *fp, ConfigArgs *c);
 static void fp_getline_init(ConfigArgs *c);
 
-static char	*strtok_quote(char *line, char *sep, char **quote_ptr);
+static char	*strtok_quote(char *line, char *sep, char **quote_ptr, int *inquote);
 static char *strtok_quote_ldif(char **line);
 
 ConfigArgs *
@@ -133,7 +133,7 @@ int config_check_vals(ConfigTable *Conf, ConfigArgs *c, int check_only ) {
 	int rc, arg_user, arg_type, arg_syn, iarg;
 	unsigned uiarg;
 	long larg;
-	unsigned long ularg;
+	size_t ularg;
 	ber_len_t barg;
 	
 	if(Conf->arg_type == ARG_IGNORED) {
@@ -628,7 +628,7 @@ strtok_quote_ldif( char **line )
 	return beg;
 }
 
-static void
+void
 config_parse_ldif( ConfigArgs *c )
 {
 	char *next;
@@ -731,6 +731,7 @@ read_config_file(const char *fname, int depth, ConfigArgs *cf, ConfigTable *cft)
 		Debug(LDAP_DEBUG_ANY,
 		    "could not stat config file \"%s\": %s (%d)\n",
 		    fname, strerror(errno), errno);
+		ch_free( c->argv );
 		ch_free( c );
 		return(1);
 	}
@@ -740,6 +741,7 @@ read_config_file(const char *fname, int depth, ConfigArgs *cf, ConfigTable *cft)
 		Debug(LDAP_DEBUG_ANY,
 		    "regular file expected, got \"%s\"\n",
 		    fname, 0, 0 );
+		ch_free( c->argv );
 		ch_free( c );
 		return(1);
 	}
@@ -750,6 +752,7 @@ read_config_file(const char *fname, int depth, ConfigArgs *cf, ConfigTable *cft)
 		Debug(LDAP_DEBUG_ANY,
 		    "could not open config file \"%s\": %s (%d)\n",
 		    fname, strerror(errno), errno);
+		ch_free( c->argv );
 		ch_free( c );
 		return(1);
 	}
@@ -2130,7 +2133,7 @@ done:;
 
 
 static char *
-strtok_quote( char *line, char *sep, char **quote_ptr )
+strtok_quote( char *line, char *sep, char **quote_ptr, int *iqp )
 {
 	int		inquote;
 	char		*tmp;
@@ -2180,6 +2183,7 @@ strtok_quote( char *line, char *sep, char **quote_ptr )
 			break;
 		}
 	}
+	*iqp = inquote;
 
 	return( tmp );
 }
@@ -2265,24 +2269,27 @@ config_fp_parse_line(ConfigArgs *c)
 		"dbpasswd",  /* in back-sql */
 		NULL
 	};
+	static char *const raw[] = {
+		"attributetype", "objectclass", "ditcontentrule", "ldapsyntax", NULL };
 	char *quote_ptr;
 	int i = (int)(sizeof(hide)/sizeof(hide[0])) - 1;
+	int inquote = 0;
 
 	c->tline = ch_strdup(c->line);
-	token = strtok_quote(c->tline, " \t", &quote_ptr);
+	token = strtok_quote(c->tline, " \t", &quote_ptr, &inquote);
 
 	if(token) for(i = 0; hide[i]; i++) if(!strcasecmp(token, hide[i])) break;
 	if(quote_ptr) *quote_ptr = ' ';
-	Debug(LDAP_DEBUG_CONFIG, "line %d (%s%s)\n", c->lineno,
+	Debug(LDAP_DEBUG_CONFIG, "%s (%s%s)\n", c->log,
 		hide[i] ? hide[i] : c->line, hide[i] ? " ***" : "");
 	if(quote_ptr) *quote_ptr = '\0';
 
-	for(;; token = strtok_quote(NULL, " \t", &quote_ptr)) {
+	for(;; token = strtok_quote(NULL, " \t", &quote_ptr, &inquote)) {
 		if(c->argc >= c->argv_size) {
 			char **tmp;
 			tmp = ch_realloc(c->argv, (c->argv_size + ARGS_STEP) * sizeof(*c->argv));
 			if(!tmp) {
-				Debug(LDAP_DEBUG_ANY, "line %d: out of memory\n", c->lineno, 0, 0);
+				Debug(LDAP_DEBUG_ANY, "%s: out of memory\n", c->log, 0, 0);
 				return -1;
 			}
 			c->argv = tmp;
@@ -2293,6 +2300,13 @@ config_fp_parse_line(ConfigArgs *c)
 		c->argv[c->argc++] = token;
 	}
 	c->argv[c->argc] = NULL;
+	if (inquote) {
+		/* these directives parse c->line independently of argv tokenizing */
+		for(i = 0; raw[i]; i++) if (!strcasecmp(c->argv[0], raw[i])) return 0;
+
+		Debug(LDAP_DEBUG_ANY, "%s: unterminated quoted string \"%s\"\n", c->log, c->argv[c->argc-1], 0);
+		return -1;
+	}
 	return(0);
 }
 
