@@ -1,7 +1,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2015 The OpenLDAP Foundation.
+ * Copyright 1998-2020 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -822,7 +822,7 @@ int get_ctrls(
 
 		} else if( c->ldctl_oid == NULL ) {
 			Debug( LDAP_DEBUG_TRACE,
-				"get_ctrls: conn %lu got emtpy OID.\n",
+				"get_ctrls: conn %lu got empty OID.\n",
 				op->o_connid, 0, 0 );
 
 			slap_free_ctrls( op, op->o_ctrls );
@@ -1392,6 +1392,7 @@ static int parseAssert (
 		}
 		if( op->o_assertion != NULL ) {
 			filter_free_x( op, op->o_assertion, 1 );
+			op->o_assertion = NULL;
 		}
 		return rs->sr_err;
 	}
@@ -1659,8 +1660,12 @@ static int parseDomainScope (
 		return LDAP_PROTOCOL_ERROR;
 	}
 
-	if ( BER_BVISNULL( &ctrl->ldctl_value )) {
-		rs->sr_text = "domainScope control value not empty";
+	/* this should be checking BVISNULL, but M$ clients are broken
+	 * and include the value even though the M$ spec says it must be
+	 * omitted. ITS#9100.
+	 */
+	if ( !BER_BVISEMPTY( &ctrl->ldctl_value )) {
+		rs->sr_text = "domainScope control value not absent";
 		return LDAP_PROTOCOL_ERROR;
 	}
 
